<?php

namespace Pinpoint\CarouselManager\Controller\Adminhtml\Carousel;

use Magento\Backend\App\Action;
use Pinpoint\CarouselManager\Model\Carousel;
use Pinpoint\CarouselManager\Model\CarouselRepository;
use Pinpoint\CarouselManager\Model\Page;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Exception\LocalizedException;

/**
 * Class Save
 * @package Pinpoint\CarouselManager\Controller\Adminhtml\Carousel
 */
class Save extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Pinpoint_CarouselManager::carousel';

    /**
     * @var DataPersistorInterface
     */
    protected $dataPersistor;

    /**
     * @var CarouselRepository
     */
    protected $carouselRespository;

    /**
     * @param Action\Context $context
     * @param DataPersistorInterface $dataPersistor
     * @param Carousel $carousel
     */
    public function __construct(
        Action\Context $context,
        DataPersistorInterface $dataPersistor,
        CarouselRepository $carouselRespository
    )
    {
        parent::__construct($context);
        $this->dataPersistor = $dataPersistor;
        $this->carouselRespository = $carouselRespository;
    }

    /**
     * Save action
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {

        $data = $this->getRequest()->getPostValue();
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        if ($data) {
            if (empty($data['carousel_id'])) {
                $data['carousel_id'] = null;
            }

            $id = $this->getRequest()->getParam('carousel_id');
            if ($id) {
                $model = $this->carouselRespository->getById($id);
            } else {
                $model = $this->carouselRespository->create();
            }
            
            $model->setData($data);

            try {
                $this->carouselRespository->save($model);
                $this->messageManager->addSuccessMessage(__('Carousel successfully saved.'));
                $this->dataPersistor->clear('pinpoint_carouselmanager_carousel');
                if ($this->getRequest()->getParam('back')) {
                    return $resultRedirect->setPath('*/*/edit', ['carousel_id' => $model->getId(), '_current' => true]);
                }
                return $resultRedirect->setPath('*/*/');
            } catch (LocalizedException $e) {
                $this->messageManager->addErrorMessage($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addExceptionMessage($e, __('Something went wrong while saving the data.'));
            }

            $this->dataPersistor->set('pinpoint_carouselmanager_carousel', $data);
            return $resultRedirect->setPath('*/*/edit', ['carousel_id' => $this->getRequest()->getParam('pinpoint_carouselmanager_carousel_id')]);
        }
        return $resultRedirect->setPath('*/*/');
    }
}
