<?php
namespace Pinpoint\CarouselManager\Model;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Api\SortOrderBuilder;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\DataObject\IdentityInterface;
use Magento\Framework\Model\AbstractModel;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Registry;
use Pinpoint\CarouselManager\Api\Data\CarouselInterface;
use Pinpoint\CarouselManager\Model\SlideFactory;
use Pinpoint\CarouselManager\Model\SlideRepository;

/**
 * Class Carousel
 * @package Pinpoint\CarouselManager\Model
 */
class Carousel extends AbstractModel implements CarouselInterface, IdentityInterface
{
    /**
     *
     */
    const CACHE_TAG = 'pinpoint_carousel';
    /**
     * @var SlideFactory
     */
    protected $slideFactory;
    /**
     * @var SlideRepository
     */
    protected $slideRespository;
    /**
     * @var SearchCriteriaBuilder
     */
    protected $searchCriteriaBuilder;
    /**
     * @var SortOrderBuilder
     */
    protected $sortOrderBuilder;
    /**
     * @var
     */
    protected $slides;

    /**
     * Carousel constructor.
     * @param Context $context
     * @param Registry $registry
     * @param SlideFactory $slideFactory
     * @param SlideRepository $slideRespository
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param SortOrderBuilder $sortOrderBuilder
     * @param AbstractResource|null $resource
     * @param AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        Context $context,
        Registry $registry,
        SlideFactory $slideFactory,
        SlideRepository $slideRespository,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        SortOrderBuilder $sortOrderBuilder,
        AbstractResource $resource = null,
        AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);

        $this->slideFactory = $slideFactory;
        $this->slideRespository = $slideRespository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->sortOrderBuilder = $sortOrderBuilder;
    }

    /**
     *
     */
    protected function _construct()
    {
        $this->_init('Pinpoint\CarouselManager\Model\ResourceModel\Carousel');
    }

    /**
     * @return array
     */
    public function getIdentities()
    {
        return [self::CACHE_TAG . '_' . $this->getId()];
    }

    /**
     * @return mixed
     */
    public function beforeSave()
    {
        $storeIds = $this->getStoreIds();
        if ($storeIds && is_array($storeIds)) {
            $storeIds = implode(',', $storeIds);
            $this->setStoreIds($storeIds);
        }
        return parent::beforeSave();
    }

    /**
     * @return mixed
     */
    public function afterLoad()
    {
        if ($storeIds = $this->getStoreIds()) {
            $this->setStoreIds(explode(',', $storeIds));
        }
        return parent::afterLoad();
    }


    /**
     * @return mixed
     */
    public function afterSave()
    {
        $data = $this->getData();
        $slidesData = isset($data['slide']) ? $data['slide'] : null;
        if ($slidesData) {
            foreach ($slidesData as $key => $item) {
                if ($key === '%s') {
                    continue;
                }

                $model = $this->slideFactory->create();
                $id = isset($item['id']) ? $item['id'] : 0;

                if (!empty($id)) {
                    $model->load($id);
                }

                if (isset($item['imageUrl'])) {
                    $item['image'] = $item['imageUrl'];
                }

                if (isset($item['imageUrlMobile'])) {
                    $item['mobile_image'] = $item['imageUrlMobile'];
                }

                $model->setData($item);
                $model->setCarouselId($this->getId());
                if (empty($model->getId())) {
                    $model->setId(null);
                }
                try {
                    $model->save();
                } Catch (\Exception $e) {
                    echo 'Exception';
                }

            }
        }

        return parent::afterSave();
    }

    /**
     * @return mixed
     */
    public function getSlides()
    {
        if (!$this->slides && $this->getId()) {

            $sortOrder = $this->sortOrderBuilder
                ->setField('sort')
                ->setDirection(\Magento\Framework\Api\SortOrder::SORT_ASC)
                ->create();

            $searchCriteria = $this->searchCriteriaBuilder
                ->addFilter('carousel_id', $this->getId(), 'eq')
                ->addSortOrder($sortOrder)
                ->create();

            $this->slides = $this->slideRespository->getList($searchCriteria)->getItems();
        }

        return $this->slides;
    }

}
